# Monkey patch function to bypass finders cache if we are using request based
# layer retrieval. This is required because the finders cache is different for
# each layer.

# todo: find a way to cache it again

import logging

from django.utils.module_loading import import_string
from django.conf import settings
from django.contrib.staticfiles import finders

logger = logging.getLogger("logger")


def my_get_finder(import_path):
    Finder = import_string(import_path)
    if not issubclass(Finder, finders.BaseFinder):
        raise ImproperlyConfigured('Finder "%s" is not a subclass of "%s"' %
                                   (Finder, finders.BaseFinder))
    return Finder()

my_get_finder.cache_clear = lambda: True


def apply_monkey(force=False):
    # Monkey need only be applied is layer lookup is from the request
    setting = getattr(settings, "LAYERS", {})
    if ("current" not in setting) or ("layers" not in setting) \
        or force:
        logger.info("Patching django.contrib.staticfiles.finders.get_finder")
        finders.get_finder = my_get_finder

apply_monkey()



import os

from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.conf import settings

from layers import reset_layer_stacks
from layers.monkey import apply_monkey


BASIC_LAYERS_ = {"tree": ["basic", ["web"]], "current": "basic"}
WEB_LAYERS_ = {"tree": ["basic", ["web"]], "current": "web"}


class DecoratorFromSettingsTestCase(TestCase):

    def setUp(self):
        super(DecoratorFromSettingsTestCase, self).setUp()
        reset_layer_stacks()
        apply_monkey(force=True)

    @override_settings(LAYERS=BASIC_LAYERS_)
    def test_exclude_from_layers_basic(self):
        url = reverse("normal-view")
        response = self.client.get(url)
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This is a normal view" in result)

        url = reverse("web-only-view")
        response = self.client.get(url)
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("is not available for your device" in result)

    @override_settings(LAYERS=WEB_LAYERS_)
    def test_exclude_from_layers_web(self):
        url = reverse("normal-view")
        response = self.client.get(url)
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This is a normal view" in result)

        url = reverse("web-only-view")
        response = self.client.get(url)
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This view is only available for web" in result)


class DecoratorFromRequestTestCase(TestCase):

    def setUp(self):
        super(DecoratorFromRequestTestCase, self).setUp()
        reset_layer_stacks()
        apply_monkey()

    @override_settings(LAYERS=BASIC_LAYERS_)
    def test_exclude_from_layers_basic(self):
        url = reverse("normal-view")
        response = self.client.get(url, **{"X-Django-Layer": "basic"})
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This is a normal view" in result)

        url = reverse("web-only-view")
        response = self.client.get(url, **{"X-Django-Layer": "basic"})
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("is not available for your device" in result)

    @override_settings(LAYERS=WEB_LAYERS_)
    def test_exclude_from_layers_web(self):
        url = reverse("normal-view")
        response = self.client.get(url, **{"X-Django-Layer": "web"})
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This is a normal view" in result)

        url = reverse("web-only-view")
        response = self.client.get(url, **{"X-Django-Layer": "web"})
        result = response.content
        self.assertEqual(response.status_code, 200)
        self.failUnless("This view is only available for web" in result)



import os
import glob


BASE_DIR = os.path.join(
    glob.glob(os.environ["VIRTUAL_ENV"] +  "/lib/*/site-packages")[0],
    "layers"
)

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "layers.db"
    }
}

ROOT_URLCONF = "layers.tests.urls"

INSTALLED_APPS = (
    "layers",
    "layers.tests",
    "layers.tests.someapp",
    "crum",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.staticfiles",
)

SECRET_KEY = "SECRET_KEY"

MIDDLEWARE_CLASSES = (
    "django.middleware.common.CommonMiddleware",
    "crum.CurrentRequestUserMiddleware",
)

# Layers loaders must precede default equivalent loaders
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            # insert your TEMPLATE_DIRS here
            os.path.join(BASE_DIR, "tests", "templates")
        ],
        "OPTIONS": {
            "context_processors": [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven"t customized them:
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
            "loaders": [
                # insert your TEMPLATE_LOADERS here
                "layers.loaders.filesystem.Loader",
                "django.template.loaders.filesystem.Loader",
                "layers.loaders.app_directories.Loader",
                "django.template.loaders.app_directories.Loader",
            ]
        },
    },
]

STATICFILES_FINDERS = (
    "layers.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "layers.finders.AppDirectoriesFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
)
STATICFILES_DIRS = (os.path.join(BASE_DIR, "tests", "static"),)

STATIC_URL = "/"
STATIC_ROOT = os.path.join(BASE_DIR, "tmp")

LAYERS = {"layers": ["basic"]}



from django.test import TestCase
from django.core.management import call_command

from layers import reset_layer_stacks, build_layer_stacks
from layers.models import Layer


class ManagementCommandsTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        super(ManagementCommandsTestCase, cls).setUpTestData()
        # Reset the state because other tests affect them
        reset_layer_stacks()
        build_layer_stacks()
        call_command("load_layers")

    def test_layer_objects(self):
        layers = [o.name for o in Layer.objects.all().order_by("name")]
        self.assertEqual(layers, ["basic"])



from django.template.utils import get_app_template_dirs

from layers.loaders.filesystem import Loader as FilesystemLoader


class Loader(FilesystemLoader):

    def get_dirs(self):
        return get_app_template_dirs("templates")



import os

from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core import management
from django.core.files.storage import default_storage
from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils.functional import empty

from layers import get_layer_stacks


class Command(BaseCommand):
    help = """Run collectstatic for each layer. You must use the new style \
LAYERS format for this to work."""

    @transaction.atomic
    def handle(self, *args, **options):
        original_static_root = settings.STATIC_ROOT

        for layer in get_layer_stacks().keys():

            # We may safely manipulate settings because management commands run
            # then exit.
            settings.LAYERS["current"] = layer
            settings.STATIC_ROOT = os.path.join(original_static_root, layer)

            # Re-initialize storage by dropping to low-level API
            default_storage._wrapped = empty
            staticfiles_storage._wrapped = empty

            management.call_command("collectstatic", interactive=False)



from django.core.management.base import BaseCommand
from django.db import transaction

from layers import get_layer_stacks, reset_layer_stacks, build_layer_stacks
from layers.models import Layer


class Command(BaseCommand):
    help = "Create objects from layers setting."

    @transaction.atomic
    def handle(self, *args, **options):
        reset_layer_stacks()
        build_layer_stacks()
        for layers in get_layer_stacks().values():
            for layer in layers:
                Layer.objects.get_or_create(name=layer)



import os

from django.core.files.storage import FileSystemStorage
from django.contrib.staticfiles.finders import FileSystemFinder as \
    BaseFileSystemFinder, AppDirectoriesFinder as BaseAppDirectoriesFinder
from django.conf import settings

from crum import get_current_request

from layers import get_current_layer_stack


class FileSystemFinder(BaseFileSystemFinder):

    def __init__(self, apps=None, *args, **kwargs):
        super(FileSystemFinder, self).__init__(apps, *args, **kwargs)

        # Extend locations
        layers = list(get_current_layer_stack(get_current_request()))
        layers.reverse()
        processed = []
        new_locations = []
        for prefix, location in self.locations:
            for layer in layers:
                pth = os.path.join(location, layer)
                if os.path.exists(pth) and (pth not in processed):
                    processed.append(pth)
                    new_locations.append(('', pth))
        for location in reversed(new_locations):
            self.locations.insert(0, location)

        for prefix, location in new_locations:
            filesystem_storage = FileSystemStorage(location=location)
            filesystem_storage.prefix = prefix
            self.storages[location] = filesystem_storage


class AppDirectoriesFinder(BaseAppDirectoriesFinder):

    def __init__(self, apps=None, *args, **kwargs):
        super(AppDirectoriesFinder, self).__init__(apps, *args, **kwargs)

        # Extends apps, add to storages
        layers = list(get_current_layer_stack(get_current_request()))
        layers.reverse()
        processed = []
        for k, v in self.storages.items():
            for layer in layers:
                pth = os.path.join(v.location, layer)
                if os.path.exists(pth) and (pth not in processed):
                    processed.append(pth)
                    # We're not really adding an app, but the superclass
                    # attaches no app specific logic to it so it is safe.
                    self.apps.append(pth)
                    filesystem_storage = FileSystemStorage(location=pth)
                    filesystem_storage.prefix = ""
                    self.storages[pth] = filesystem_storage



from django.db import models

from layers import monkey


class Layer(models.Model):
    """A model to represent a layer. This exists purely to facilitate filtering
    at database level."""
    name = models.CharField(max_length=64, unique=True)

    class Meta:
        ordering = ("name",)

    def __unicode__(self):
        return self.name



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2016-12-19 10:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('layers', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='layer',
            options={'ordering': ('name',)},
        ),
    ]



import os

from django.template.loaders.filesystem import Loader as BaseLoader
from django.template import Origin
from django.utils._os import safe_join
from django.core.exceptions import SuspiciousFileOperation
from django.conf import settings

from crum import get_current_request

from layers import get_current_layer_stack


class Loader(BaseLoader):

    def get_template_sources(self, template_name, template_dirs=None):
        """Make the loader layer aware"""

        if not template_dirs:
            template_dirs = self.get_dirs()
        layers = list(get_current_layer_stack(get_current_request()))
        layers.reverse()
        for template_dir in template_dirs:
            for layer in layers:
                l_template_name = os.path.join(layer, template_name)
                try:
                    name = safe_join(template_dir, l_template_name)
                except SuspiciousFileOperation:
                    # The joined path was located outside of this template_dir
                    # (it might be inside another one, so this isn"t fatal).
                    continue

                yield Origin(
                    name=name,
                    template_name=template_name,
                    loader=self,
                )




from functools import wraps

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.decorators import available_attrs
from django.conf import settings

from layers import get_current_layer


def exclude_from_layers(layers):

    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(view_or_request, *args, **kwargs):
            request = getattr(view_or_request, "request", view_or_request)
            layer = get_current_layer(request)

            # Redirect if view not available in this layer
            if layer and (layer in layers):
                return render_to_response(
                    "layers/exclude_from_layers.html",
                    {"layer": layer, "request": request}
                )
                return response

            return view_func(view_or_request, *args, **kwargs)

        return _wrapped_view
    return decorator



import os
from shutil import rmtree

from django.template import RequestContext, loader
from django.core.urlresolvers import reverse
from django.core import management
from django.core.files.storage import default_storage
from django.utils.functional import empty
from django.contrib.staticfiles.storage import staticfiles_storage
from django.contrib.staticfiles import finders
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.conf import settings

from crum import _thread_locals

from layers import reset_layer_stacks
from layers.monkey import apply_monkey


BASIC_LAYERS_FROM_SETTINGS = {"tree": ["basic", ["web"]], "current": "basic"}
WEB_LAYERS_FROM_SETTINGS = {"tree": ["basic", ["web"]], "current": "web"}
BASIC_LAYERS_FROM_REQUEST = {"tree": ["basic", ["web"]], "test-x-django-layer": "basic"}
WEB_LAYERS_FROM_REQUEST = {"tree": ["basic", ["web"]], "test-x-django-layer": "web"}


class LayerFromSettingsTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super(LayerFromSettingsTestCase, cls).setUpClass()
        cls.factory = RequestFactory()

    def setUp(self):
        super(LayerFromSettingsTestCase, self).setUp()
        reset_layer_stacks()
        apply_monkey(force=True)

    def get_rendered_template(self, template_name):
        # Check is settings specify a test request header
        layer = settings.LAYERS.get("test-x-django-layer", None)
        if layer:
            request = self.factory.get("/", **{"X-Django-Layer": layer})
        else:
            request = self.factory.get("/")

        # Manually set the request for crum since these tests are at template
        # level, ie. without a request as would be the case in the real world.
        _thread_locals.request = request

        template = loader.get_template(template_name)
        return template.render(RequestContext(request)).strip()

    def get_rendered_static(self, static_name):
        absolute_path = finders.find(static_name)
        f = open(absolute_path, "r")
        content = f.read()
        f.close()
        return content.strip()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_plain_template_basic(self):
        content = self.get_rendered_template("tests/plain.html")
        self.assertEqual(content, "fs plain")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_plain_template_web(self):
        content = self.get_rendered_template("tests/plain.html")
        self.assertEqual(content, "fs plain")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_plain_template_basic(self):
        content = self.get_rendered_template("someapp/plain.html")
        self.assertEqual(content, "app plain")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_plain_template_web(self):
        content = self.get_rendered_template("someapp/plain.html")
        self.assertEqual(content, "app plain")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_foo_template_basic(self):
        content = self.get_rendered_template("tests/foo.html")
        self.assertEqual(content, "fs foo basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_foo_template_web(self):
        content = self.get_rendered_template("tests/foo.html")
        self.assertEqual(content, "fs foo basic")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_foo_template_basic(self):
        content = self.get_rendered_template("someapp/foo.html")
        self.assertEqual(content, "app foo basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_foo_template_web(self):
        content = self.get_rendered_template("someapp/foo.html")
        self.assertEqual(content, "app foo basic")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_bar_template_basic(self):
        content = self.get_rendered_template("tests/bar.html")
        self.assertEqual(content, "fs bar basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_bar_template_web(self):
        content = self.get_rendered_template("tests/bar.html")
        self.assertEqual(content, "fs bar web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_bar_template_basic(self):
        content = self.get_rendered_template("someapp/bar.html")
        self.assertEqual(content, "app bar basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_bar_template_web(self):
        content = self.get_rendered_template("someapp/bar.html")
        self.assertEqual(content, "app bar web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_plain_static_basic(self):
        content = self.get_rendered_static("tests/plain.css")
        self.assertEqual(content, "fs plain")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_plain_static_web(self):
        content = self.get_rendered_static("tests/plain.css")
        self.assertEqual(content, "fs plain")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_plain_static_basic(self):
        content = self.get_rendered_static("someapp/plain.css")
        self.assertEqual(content, "app plain")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_plain_static_web(self):
        content = self.get_rendered_static("someapp/plain.css")
        self.assertEqual(content, "app plain")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_foo_static_basic(self):
        content = self.get_rendered_static("tests/foo.css")
        self.assertEqual(content, "fs foo basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_foo_static_web(self):
        content = self.get_rendered_static("tests/foo.css")
        self.assertEqual(content, "fs foo basic")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_foo_static_basic(self):
        content = self.get_rendered_static("someapp/foo.css")
        self.assertEqual(content, "app foo basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_foo_static_web(self):
        content = self.get_rendered_static("someapp/foo.css")
        self.assertEqual(content, "app foo basic")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_bar_static_basic(self):
        content = self.get_rendered_static("tests/bar.css")
        self.assertEqual(content, "fs bar basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_bar_static_web(self):
        content = self.get_rendered_static("tests/bar.css")
        self.assertEqual(content, "fs bar web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_bar_static_basic(self):
        content = self.get_rendered_static("someapp/bar.css")
        self.assertEqual(content, "app bar basic")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_bar_static_web(self):
        content = self.get_rendered_static("someapp/bar.css")
        self.assertEqual(content, "app bar web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_plain_override_me_static_basic(self):
        # Basic does not override the file defined unaware of layers
        content = self.get_rendered_static("tests/plain_override_me.css")
        self.assertEqual(content, "fs plain override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_plain_override_me_static_web(self):
        # Web does override the file defined unaware of layers
        content = self.get_rendered_static("tests/plain_override_me.css")
        self.assertEqual(content, "fs plain override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_basic_override_me_static_basic(self):
        # Basic defines the file first
        content = self.get_rendered_static("tests/basic_override_me.css")
        self.assertEqual(content, "fs basic override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_basic_override_me_static_web(self):
        # Web does override the file defined in basic
        content = self.get_rendered_static("tests/basic_override_me.css")
        self.assertEqual(content, "fs basic override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_plain_override_me_template_basic(self):
        # Basic does not override the file defined unaware of layers
        content = self.get_rendered_template("tests/plain_override_me.html")
        self.assertEqual(content, "fs plain override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_plain_override_me_template_web(self):
        # Web does override the file defined unaware of layers
        content = self.get_rendered_template("tests/plain_override_me.html")
        self.assertEqual(content, "fs plain override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_fs_basic_override_me_template_basic(self):
        # Basic defines the file first
        content = self.get_rendered_template("tests/basic_override_me.html")
        self.assertEqual(content, "fs basic override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_fs_basic_override_me_template_web(self):
        # Web does override the file defined in basic
        content = self.get_rendered_template("tests/basic_override_me.html")
        self.assertEqual(content, "fs basic override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_plain_override_me_template_basic(self):
        # Basic does not override the file defined unaware of layers
        content = self.get_rendered_template("someapp/plain_override_me.html")
        self.assertEqual(content, "app plain override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_plain_override_me_template_web(self):
        # Web does override the file defined unaware of layers
        content = self.get_rendered_template("someapp/plain_override_me.html")
        self.assertEqual(content, "app plain override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_basic_override_me_template_basic(self):
        # Basic defines the file first
        content = self.get_rendered_template("someapp/basic_override_me.html")
        self.assertEqual(content, "app basic override me")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_basic_override_me_template_web(self):
        # Web does override the file defined in basic
        content = self.get_rendered_template("someapp/basic_override_me.html")
        self.assertEqual(content, "app basic override me web")

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS)
    def test_app_basic_override_me_in_another_app_template_basic(self):
        # Basic defines the file first
        content = self.get_rendered_template("someapp/basic_override_me_in_another_app.html")
        self.assertEqual(content, "app basic override me in another app")

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS)
    def test_app_basic_override_me_in_another_app_template_web(self):
        # Web does override the file defined in basic. Crucially it is
        # overridden by the "tests" app.
        content = self.get_rendered_template("someapp/basic_override_me_in_another_app.html")
        self.assertEqual(content, "app basic overridden in tests web")


class CollectStaticTestCase(LayerFromSettingsTestCase):

    def setUp(self):
        super(CollectStaticTestCase, self).setUp()
        try:
            rmtree("/tmp/django-layers")
        except OSError:
            pass

    @override_settings(LAYERS=BASIC_LAYERS_FROM_SETTINGS, STATIC_ROOT="/tmp/django-layers/static/basic")
    def test_collectstatic_basic(self):

        # Re-initialize storage by dropping to low-level API
        default_storage._wrapped = empty
        staticfiles_storage._wrapped = empty

        management.call_command("collectstatic", interactive=False)
        for name, exists, content in (
            (("someapp/bar.css",), True, "app bar basic"),
            (("someapp/foo.css",), True, "app foo basic"),
            (("someapp/plain.css",), True, "app plain"),
            (("tests/bar.css",), True, "fs bar basic"),
            (("tests/basic.css",), True, "fs basic"),
            (("tests/basic_override_me.css",), True, "fs basic override me"),
            (("tests/foo.css",), True, "fs foo basic"),
            (("tests/plain.css",), True, "fs plain"),
            (("tests/plain_override_me.css",), True, "fs plain override me"),
            (("tests/web.css",), False, "fs web"),
        ):
            pth = os.path.join(settings.STATIC_ROOT, *name)
            self.assertEqual(os.path.exists(pth), exists)
            if exists:
                self.assertEqual(open(pth).read().strip(), content)

    @override_settings(LAYERS=WEB_LAYERS_FROM_SETTINGS, STATIC_ROOT="/tmp/django-layers/static/web")
    def test_collectstatic_web(self):

        # Re-initialize storage by dropping to low-level API
        default_storage._wrapped = empty
        staticfiles_storage._wrapped = empty

        management.call_command("collectstatic", interactive=False)
        for name, exists, content in (
            (("someapp/bar.css",), True, "app bar web"),
            (("someapp/foo.css",), True, "app foo basic"),
            (("someapp/plain.css",), True, "app plain"),
            (("tests/bar.css",), True, "fs bar web"),
            (("tests/basic.css",), True, "fs basic"),
            (("tests/basic_override_me.css",), True, "fs basic override me web"),
            (("tests/foo.css",), True, "fs foo basic"),
            (("tests/plain.css",), True, "fs plain"),
            (("tests/plain_override_me.css",), True, "fs plain override me web"),
            (("tests/web.css",), True, "fs web"),
        ):
            pth = os.path.join(settings.STATIC_ROOT, *name)
            self.assertEqual(os.path.exists(pth), exists)
            if exists:
                self.assertEqual(open(pth).read().strip(), content)


class LayerFromRequestTestCase(LayerFromSettingsTestCase):

    def setUp(self):
        # Override because no need to force the monkey
        super(LayerFromRequestTestCase, self).setUp()
        reset_layer_stacks()
        apply_monkey()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_plain_template_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_plain_template_web(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_plain_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_plain_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_plain_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_plain_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_foo_template_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_foo_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_foo_template_web(self):
        super(LayerFromRequestTestCase, self).test_fs_foo_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_foo_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_foo_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_foo_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_foo_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_bar_template_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_bar_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_bar_template_web(self):
        super(LayerFromRequestTestCase, self).test_fs_bar_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_bar_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_bar_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_bar_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_bar_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_plain_static_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_plain_static_web(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_plain_static_basic(self):
        super(LayerFromRequestTestCase, self).test_app_plain_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_plain_static_web(self):
        super(LayerFromRequestTestCase, self).test_app_plain_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_foo_static_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_foo_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_foo_static_web(self):
        super(LayerFromRequestTestCase, self).test_fs_foo_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_foo_static_basic(self):
        super(LayerFromRequestTestCase, self).test_app_foo_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_foo_static_web(self):
        super(LayerFromRequestTestCase, self).test_app_foo_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_bar_static_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_bar_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_bar_static_web(self):
        super(LayerFromRequestTestCase, self).test_fs_bar_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_bar_static_basic(self):
        super(LayerFromRequestTestCase, self).test_app_bar_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_bar_static_web(self):
        super(LayerFromRequestTestCase, self).test_app_bar_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_plain_override_me_static_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_override_me_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_plain_override_me_static_web(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_override_me_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_basic_override_me_static_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_basic_override_me_static_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_basic_override_me_static_web(self):
        super(LayerFromRequestTestCase, self).test_fs_basic_override_me_static_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_plain_override_me_template_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_override_me_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_plain_override_me_template_web(self):
        super(LayerFromRequestTestCase, self).test_fs_plain_override_me_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_fs_basic_override_me_template_basic(self):
        super(LayerFromRequestTestCase, self).test_fs_basic_override_me_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_fs_basic_override_me_template_web(self):
        super(LayerFromRequestTestCase, self).test_fs_basic_override_me_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_plain_override_me_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_plain_override_me_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_plain_override_me_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_plain_override_me_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_basic_override_me_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_basic_override_me_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_basic_override_me_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_basic_override_me_template_web()

    @override_settings(LAYERS=BASIC_LAYERS_FROM_REQUEST)
    def test_app_basic_override_me_in_another_app_template_basic(self):
        super(LayerFromRequestTestCase, self).test_app_basic_override_me_in_another_app_template_basic()

    @override_settings(LAYERS=WEB_LAYERS_FROM_REQUEST)
    def test_app_basic_override_me_in_another_app_template_web(self):
        super(LayerFromRequestTestCase, self).test_app_basic_override_me_in_another_app_template_web()



import multiprocessing
from setuptools import setup, find_packages


setup(
    name="django-layers-hr",
    version="1.10.1",
    description="Serve different templates and static files for eg. mobi and web. Layers can be stacked to enable resource re-use.",
    long_description = open("README.rst", "r").read() + open("AUTHORS.rst", "r").read() + open("CHANGELOG.rst", "r").read(),
    author="Hedley Roos",
    author_email="hedleyroos@gmail.com",
    license="BSD",
    url="http://github.com/hedleyroos/django-layers",
    packages = find_packages(),
    install_requires = [
        "django",
        "django-crum"
    ],
    include_package_data=True,
    tests_require=[
        "tox",
    ],
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



from django.conf.urls import include, url

from layers.tests import views


urlpatterns = [
    url(
        r"^normal-view/$",
        views.NormalView.as_view(),
        name="normal-view"
    ),
    url(
        r"^web-only-view/$",
        views.WebOnlyView.as_view(),
        name="web-only-view"
    )
]



from django.views.generic.base import TemplateView

from layers.decorators import exclude_from_layers


class NormalView(TemplateView):
    template_name = "tests/normal_view.html"


class WebOnlyView(TemplateView):
    template_name = "tests/web_only_view.html"

    @exclude_from_layers(layers=("basic",))
    def get(self, *args, **kwargs):
        return super(WebOnlyView, self).get(*args, **kwargs)



import types

from django.conf import settings


default_app_config = "layers.apps.LayersConfig"
LAYER_STACKS = {}


def _build_layer_stacks(node, di, parent=None, depth=0):
    """Recursive helper to map each tree node to its parent."""

    if isinstance(node, types.ListType):
        for child in node:
            if isinstance(child, types.ListType):
                _build_layer_stacks(child, di, node[0], depth+1)
            else:
                _build_layer_stacks(child, di, parent, depth+1)
    else:
        if node in di:
            raise RuntimeError, "Node %s is already in the tree" % node
        di[node] = parent


def build_layer_stacks():
    """Support both the legacy "layers" format and the newer "tree" format.
    Tree uses a list of lists format, eg.
    single inheritance line basic-smart-web
       ['basic',
           ['smart',
               ['web']
           ]
       ]
    or two inheritance lines basic-smart, basic-web
       ['basic',
         ['web'],
         ['smart']
       ]
    """

    global LAYER_STACKS
    setting = getattr(settings, "LAYERS", {})
    tree = setting.get("tree", [])
    layers = setting.get("layers", [])
    if tree:
        di = {}
        _build_layer_stacks(tree, di)

        # Build a dictionary of layer -> parent layers
        for layer in di.keys():
            LAYER_STACKS[layer] = [layer]
            parent = di[layer]
            while parent is not None:
                LAYER_STACKS[layer].insert(0, parent)
                parent = di.get(parent, None)

    elif layers:
        LAYER_STACKS[layers[-1]] = layers


def get_layer_stacks():
    global LAYER_STACKS
    return LAYER_STACKS


def get_current_layer(request=None):
    """Return the current layer. The setting, if set, trumps the request."""

    current = None
    setting = getattr(settings, "LAYERS", {})
    if "current" in setting:
        current = setting["current"]
    elif "layers" in setting:
        current = setting["layers"][-1]
    elif request is not None:
        current = request.META.get("X-Django-Layer", None)
    return current


def get_current_layer_stack(request=None):
    global LAYER_STACKS
    if not LAYER_STACKS:
        build_layer_stacks()
    current = get_current_layer(request)
    return LAYER_STACKS[current]


def reset_layer_stacks():
    global LAYER_STACKS
    LAYER_STACKS = {}


build_layer_stacks()
